/*
 * BaseStation_callback.cpp
 *
 *  Created on: Aug 31, 2011
 *      Author: ra23501
 */

// includes
#include "BaseStation.hpp"

// callback for robot state update
void BaseStation::callbackStateUpdate(const boost::shared_ptr<wpi_msgs::RobotState const> &robot_state_msg)
{
  mutex_.lock();
  received_robot_update_ = true;
  robot_state_ = *robot_state_msg;
  current_UAV_ = robot_state_msg->uid;
  ROS_DEBUG("Received state update.\n");
  mutex_.unlock();
}

// callback for land request message
void BaseStation::callbackLandRequestMsg(const boost::shared_ptr<wpi_msgs::LandRequest const> &land_request_msg)
{
  mutex_.lock();
  if (UAV_landing_)
  {
    // this will only occur once per UAV as it wants to land, as it only sends out one request
    waiting_UAVs_.push_back(land_request_msg->uid);
  }
  else
  {
    received_land_request_ = true;
    current_UAV_ = land_request_msg->uid;
    UAV_landing_ = true;
  }ROS_DEBUG("Received land request.\n");
  mutex_.unlock();
}

// callback for land confirm message
void BaseStation::callbackLandConfirmMsg(const boost::shared_ptr<wpi_msgs::LandConfirm const> &land_confirm_msg)
{
  mutex_.lock();
  if (land_confirm_msg->confirm)
  {
    received_positive_land_confirm_ = true;
    ROS_DEBUG("Received positive confirm.\n");
  }
  else
  {
    received_negative_land_confirm_ = true;
    ROS_DEBUG("Received negative confirm.\n");
  }
  mutex_.unlock();
}
