/*
 * BaseStation_init.cpp
 *
 *  Created on: Aug 31, 2011
 *      Author: ra23501
 */

// includes
#include "BaseStation.hpp"

// init params
void BaseStation::initParams(void)
{
  // ros params
  private_nh_.param("uid", uid_, string("base"));
  private_nh_.param("base_thread_rate", base_thread_rate_, 20.0);
  private_nh_.param("max_battery_voltage", max_battery_voltage_, 12.6);
  private_nh_.param("min_battery_voltage", min_battery_voltage_, 9.5);
  private_nh_.param("return_battery_voltage", return_battery_voltage_, 10.5);

  // init params
  robot_state_.uid = uid_;
  robot_state_.translation = geometry_msgs::Vector3();
  robot_state_.rotation = geometry_msgs::Quaternion();
  robot_state_.rotation.w = 1.0;
  robot_state_.battery_voltage = max_battery_voltage_;
  base_state_ = LAUNCHING;

  // init volatile bools
  received_robot_update_ = true;
  received_land_request_ = false;
  received_positive_land_confirm_ = false;
  received_negative_land_confirm_ = false;
  UAV_landing_ = true;
}

// init subscribers
void BaseStation::initSubscribers(void)
{
  robot_state_sub_ = nh_.subscribe(string("/robot_state"), 10, &BaseStation::callbackStateUpdate, this);
  land_request_sub_ = nh_.subscribe(string("land_request"), 10, &BaseStation::callbackLandRequestMsg, this);
  land_confirm_sub_ = nh_.subscribe(string("land_confirm"), 10, &BaseStation::callbackLandConfirmMsg, this);
}

// init batteries
void BaseStation::initBatteries(void)
{
  int numOpen = 0;
  openStation = NUM_BATTERIES;
  for (int i = 0; i < NUM_BATTERIES; i++)
  {
    allBatteries[i] = new BatteryPack(0, 0, 0, i);
    if (controller.batteryPresent(i))
    {
      readyBatteries.push_back(allBatteries[i]);
    } else {
      openStation = i;
      allBatteries[i]->location = -1;
      numOpen++;
    }
  }
  if(openStation == NUM_BATTERIES) {
    ROS_ERROR("Too many batteries or communication error, no open station.");
  }
  if(numOpen > 1) {
    ROS_ERROR("Too few batteries or dislodged battery.");
  }
}
