/*
 * MaestroController_actions.cpp
 *
 *  Created on: Sep 20, 2011
 *      Author: ra23501
 */

#include "MaestroController.hpp"

// initialize the whole base system
// pre conditions: any
// post conditions: retract the arms, center the cart and raise the scissor lift to be flush with surface
void MaestroController::initializeSystem(void)
{
  stepsPerStation = 0;
  short serials[] = {PRIMARY_SERIAL};
  short servos[] = {ARM_SERVO_0};
  short targets[] = {1874 * 4};
  short numServo;
  ROS_ERROR("Initialize systems");
  disableAll();
  numServo = moveServos(serials, servos, targets, 1);
  ros::Duration(2).sleep();
  disableServo(PRIMARY_SERIAL, ARM_SERVO_0);
  servos[0] = ARM_SERVO_1;
  targets[0] = 1956 * 4;
  numServo = moveServos(serials, servos, targets, 1);
  ros::Duration(2).sleep();
  disableServo(PRIMARY_SERIAL, ARM_SERVO_1);
  moveScissorLift(SCISSOR_DOWN, false);
  ros::Duration(1).sleep();
  homeStepper();
  centerCart();
  moveScissorLift(SCISSOR_UAV, true);
  ros::Duration(1).sleep();
  moveScissorLift(SCISSOR_REST, false);
}

// actuate the arms to center the UAV on the platform
// pre conditions: UAV has landed inside the arms' area
// post condition: UAV is centered over the platform and is retained vertically by the arms
void MaestroController::centerUAV(void)
{
  short serials[] = {PRIMARY_SERIAL};
  short servos[] = {ARM_SERVO_0};
  short targets[] = {1170 * 4};
  short numServo;
  ROS_ERROR("center UAV");
  numServo = moveServos(serials, servos, targets, 1);
  ros::Duration(2).sleep();
  disableServo(PRIMARY_SERIAL, ARM_SERVO_0);
  servos[0] = ARM_SERVO_1;
  targets[0] = 1200 * 4;
  numServo = moveServos(serials, servos, targets, 1);
  ros::Duration(2).sleep();
  disableServo(PRIMARY_SERIAL, ARM_SERVO_1);
}

// remove the battery from the UAV, assume it always works
// pre conditions: UAV has been centered over the scissor lift
// post conditions: UAV is battery-less, spent battery is on the scissor lift, which is in the lowest state
void MaestroController::removeBatteryUAV(void)
{
  short numServo;
  if ((numServo = moveScissorLift(SCISSOR_UAV, true)) != -1)
  {
    ROS_ERROR("Servo %d stalled out.", numServo);
  }
  if ((numServo = moveScissorLift(SCISSOR_DOWN, false)) != -1)
  {
    ROS_ERROR("Servo %d stalled out.", numServo);
  }
}

// load the battery recently taken from the UAV into the open charging slot
// makes sure battery has started to charge before moving on
// pre conditions: battery is on the scissor lift and it is in the lowest state
// post conditions: battery has started charging, scissor lift is in lowest state and at one end of the track, scissor lift is empty
void MaestroController::chargeSpentBattery(char openStation)
{
  char location = getStepperLocation();
  if (location == openStation)
  {
    moveCartLeft();
  }
  /*else if (location == ((openStation + 4) % 8)) // opposite station, as cart can move to both ends
   {
   moveCartRight();
   }*/
  else
  {
    moveToStation(openStation); // optimizes for smallest movement, must check direction again
    if (location == openStation)
    {
      moveCartLeft();
    }
    /*else if (location == ((openStation + 4) % 8))
     {
     moveCartRight();
     }*/
  }
  while (startCharger(openStation) != 0) // keeps trying until charging has successfully started
  {
    if (moveScissorLift(SCISSOR_CHARGE, true) != -1)
    {
      ROS_ERROR("Servo stalled out.");
    }
    if (moveScissorLift(SCISSOR_DOWN, false) != -1)
    {
      ROS_ERROR("Servo stalled out.");
    }
  }
}

// obtain the battery that was first to be done charging
// pre conditions: scissor lift is in lowest state and is empty
// post conditions: scissor lift is in lowest state with a charged battery, cart is centered and turntable is homed
void MaestroController::getChargedBattery(char chargedStation)
{
  char location = getStepperLocation();
  if (location == chargedStation)
  {
    moveCartLeft();
  }
  /*else if (location == ((chargedStation + 4) % 8))
   {
   moveCartRight();
   }*/
  else
  {
    moveToStation(chargedStation);
    if (location == chargedStation)
    {
      moveCartLeft();
    }
    /*else if (location == ((chargedStation + 4) % 8))
     {
     moveCartRight();
     }*/
  }
  while (batteryPresent(chargedStation)) // keep trying to dislodge battery until it is no longer detected by the charger
  {
    if (moveScissorLift(SCISSOR_CHARGE, true) != -1)
    {
      ROS_ERROR("Servo stalled out.");
    }
    if (moveScissorLift(SCISSOR_DOWN, false) != -1)
    {
      ROS_ERROR("Servo stalled out.");
    }
  }
  centerCart(); // in preparation for placing in the UAV
  homeStepper(); // home stepper for correct alignment between scissor lift and upper platform
}

// place the battery in the UAV
// pre conditions: scissor lift is in lowest state with a charged battery, cart is centered, turntable is homed
// post conditions: battery is placed in UAV, scissor lift is in resting (flush with landing plate) state
void MaestroController::replaceBatteryUAV(void)
{
  // cart is already at the center, just need to lift it and then return to resting position
  // assumes that the battery engages first try, can be made to check with UAV if it actually engaged later
  if (moveScissorLift(SCISSOR_UAV, true) != -1)
  {
    ROS_ERROR("Servo stalled out.");
  }
  if (moveScissorLift(SCISSOR_REST, false) != -1)
  {
    ROS_ERROR("Servo stalled out.");
  }
}

// remove the arms from retaining the UAV in preparation for takeoff
// pre conditions: battery has been replaced in the UAV
// post conditions: retaining arms are retracted and UAV has or is in the process of taking off
void MaestroController::retractArms(void)
{
  //temporary testing code


  /* disableAll();

   ROS_ERROR("lower scissor lift");
   //  moveScissorLift(SCISSOR_DOWN, false);
   //ros::Duration(1).sleep();


   ROS_ERROR("center cart");
   // centerCart();
   disableAll();
   ROS_ERROR("raise scissor lift");
   moveScissorLift(SCISSOR_UAV, true);
   disableAll();
   // ros::Duration(2).sleep();
   // ROS_ERROR("flush scissor lift");
   // moveScissorLift(SCISSOR_REST, false);
   // disableAll();
   //  ros::Duration(5).sleep();*/

  /*homeStepper();
   ros::Duration(2).sleep();
   moveToStation(3);
   ros::Duration(2).sleep();
   moveToStation(5);*/

  moveScissorLift(SCISSOR_UAV, true);
  ros::Duration(1).sleep();
  moveScissorLift(SCISSOR_DOWN, false);
  ros::Duration(1).sleep();
  moveCartLeft();
  ros::Duration(1).sleep();
  moveScissorLift(SCISSOR_CHARGE, true);
  ros::Duration(1).sleep();
  moveScissorLift(SCISSOR_DOWN, false);
  ros::Duration(1).sleep();
  moveToStation(3);
  ros::Duration(1).sleep();
  moveScissorLift(SCISSOR_CHARGE, true);
  ros::Duration(1).sleep();
  moveScissorLift(SCISSOR_DOWN, false);
  ros::Duration(1).sleep();
  centerCart();
  ros::Duration(1).sleep();
  moveToStation(4);
  ros::Duration(1).sleep();
  moveScissorLift(SCISSOR_UAV, true);
  ros::Duration(1).sleep();
  moveScissorLift(SCISSOR_REST, false);
  ros::Duration(1).sleep();
  //end of testing code
  short serials[] = {PRIMARY_SERIAL};
  short servos[] = {ARM_SERVO_0};
  short targets[] = {1874 * 4};
  short numServo;
  ROS_ERROR("retract arms");
  numServo = moveServos(serials, servos, targets, 1);
  ros::Duration(2).sleep();
  disableServo(PRIMARY_SERIAL, ARM_SERVO_0);
  servos[0] = ARM_SERVO_1;
  targets[0] = 1956 * 4;
  numServo = moveServos(serials, servos, targets, 1);
  ros::Duration(2).sleep();
  disableServo(PRIMARY_SERIAL, ARM_SERVO_1);
}

void MaestroController::disableAll(void)
{

  disableServo(PRIMARY_SERIAL, ARM_SERVO_0);
  disableServo(PRIMARY_SERIAL, ARM_SERVO_1);
  disableServo(PRIMARY_SERIAL, CART_0_SERVO);
  disableServo(PRIMARY_SERIAL, CART_0_MOTOR);
  //disableServo(PRIMARY_SERIAL, CART_1_SERVO);

}
