/*
 * MaestroController_batteries.cpp
 *
 *  Created on: Sep 20, 2011
 *      Author: ra23501
 */

#include "MaestroController.hpp"

// checks to see if a battery is in  the charging station specified
bool MaestroController::batteryPresent(char station)
{
  short batteryHere[1];
  short serial[] = {SECONDARY_SERIAL};
  short channel[] = {BATTERY_PRESENT_0};
  switch (station)
  {
    case (0):
      channel[0] = BATTERY_PRESENT_0;
      break;
    case (1):
      channel[0] = BATTERY_PRESENT_1;
      break;
    case (2):
      channel[0] = BATTERY_PRESENT_2;
      break;
    case (3):
      channel[0] = BATTERY_PRESENT_3;
      break;
    case (4):
      channel[0] = BATTERY_PRESENT_4;
      break;
    case (5):
      channel[0] = BATTERY_PRESENT_5;
      break;
    case (6):
      channel[0] = BATTERY_PRESENT_6;
      break;
    case (7):
      channel[0] = BATTERY_PRESENT_7;
      break;
  }
  getChannelValues(serial, channel, batteryHere, 1);
  return batteryHere[0] == ON;
}

// attempts to start charging the battery in the specified station, returns -1 on no battery found
int MaestroController::startCharger(char station)
{
  ros::Rate wait(50);
  ros::Rate sample(3);
  short serial[] = {SECONDARY_SERIAL};
  short channel[] = {station};
  short LED_color[1];
  char isCharging = 0;
  bool present = batteryPresent(station - BATTERY_START_RELAY_0);
  if (present)
  {
    wait.reset();
    setTarget(serial[0], ON, channel[0]);
    wait.sleep();
    setTarget(serial[0], OFF, channel[0]);
    wait.sleep();
    sample.reset();
    for (int i = 0; i < 3; i++) // sample 3 times over one second, blink at 2Hz
    {
      getChannelValues(serial, channel, LED_color, 1);
      if (ISORANGE(LED_color[0]))
        isCharging++;
      sample.sleep();
    }
    if (isCharging == 0)
    {
      return -1; // charged or disconnected/error
    }
    else if (isCharging < 3) // LEDs will be solid orange when charging, if not solid, is still waiting, so press again
    {
      wait.reset();
      setTarget(serial[0], ON, channel[0]);
      wait.sleep();
      setTarget(serial[0], OFF, channel[0]);
    }
    return 0;
  }
  else
    return -1;
}

// polls the channels for the stop charge signal from each of the chargers
// if a battery is done charging, its station number is returned
char MaestroController::doneCharging(void)
{
  short serials[] = {SECONDARY_SERIAL, SECONDARY_SERIAL, SECONDARY_SERIAL, SECONDARY_SERIAL, SECONDARY_SERIAL,
                     SECONDARY_SERIAL, SECONDARY_SERIAL, SECONDARY_SERIAL};
  short channels[] = {BATTERY_LED_0, BATTERY_LED_1, BATTERY_LED_2, BATTERY_LED_3, BATTERY_LED_4, BATTERY_LED_5,
                      BATTERY_LED_6, BATTERY_LED_7};
  short values[NUM_BATTERIES];
  char green_check[NUM_BATTERIES];
  ros::Rate sample(3);
  for (int i = 0; i < 3; i++) // sample 3 times over one second, blink at 2Hz
  {
    getChannelValues(serials, channels, values, NUM_BATTERIES);
    for (int k = 0; k < NUM_BATTERIES; k++)
    {
      if (ISGREEN(values[k]))
      {
        if ((++green_check[k]) == 3)
        {
          return k;
        }
      }
    }
    sample.sleep();
  }
  return NUM_BATTERIES;
}
