/*
 * MaestroController_cart.cpp
 *
 *  Created on: Sep 20, 2011
 *      Author: ra23501
 */

#include "MaestroController.hpp"

// centers the battery cart, defaulting to rightwards motion if position is unknown
void MaestroController::centerCart(void)
{
  short cartPositions[2/*3*/];
  short serials[] = {PRIMARY_SERIAL, /*PRIMARY_SERIAL,*/PRIMARY_SERIAL};
  short channels[] = {CART_END_0, /*CART_END_1,*/CART_MID};
  ros::Rate delay(60);
  getChannelValues(serials, channels, cartPositions, 2/*3*/);
  if (cartPositions[0] == ON_INP) // left end
  {
    setTarget(PRIMARY_SERIAL, CART_RIGHT, CART_0_MOTOR);
    while (cartPositions[1/*2*/] != ON_INP)
    {
      getChannelValues(&(serials[1/*2*/]), &(channels[1/*2*/]), &(cartPositions[1/*2*/]), 1);
      delay.sleep();
    }
    setTarget(PRIMARY_SERIAL, STOP, CART_0_MOTOR);
    disableServo(PRIMARY_SERIAL, CART_0_MOTOR);
  }
  /*else if (cartPositions[1] == ON) // right end
   {
   setTarget(PRIMARY_SERIAL, CART_LEFT, CART_0_MOTOR);
   while (cartPositions[2] == OFF)
   {
   getChannelValues(&(serials[2]), &(channels[2]), &(cartPositions[2]), 1);
   }
   setTarget(PRIMARY_SERIAL, STOP, CART_0_MOTOR);
   }*/
  else if (cartPositions[1/*2*/] == ON_INP) // middle
  {
    return;
  }
  else
  {
    setTarget(PRIMARY_SERIAL, CART_RIGHT, CART_0_MOTOR);
    while (cartPositions[0] != ON_INP && cartPositions[1] != ON_INP/* && cartPositions[2] == OFF*/)
    {
      getChannelValues(serials, channels, cartPositions, 2/*3*/);
      delay.sleep();
    }
    setTarget(PRIMARY_SERIAL, STOP, CART_0_MOTOR);
    disableServo(PRIMARY_SERIAL, CART_0_MOTOR);
    /*if (cartPositions[1] == ON) // right end
     {
     setTarget(PRIMARY_SERIAL, CART_LEFT, CART_0_MOTOR);
     while (cartPositions[2] == OFF)
     {
     getChannelValues(&(serials[2]), &(channels[2]), &(cartPositions[2]), 1);
     }
     setTarget(PRIMARY_SERIAL, STOP, CART_0_MOTOR);
     }
     else */if (cartPositions[1/*2*/] == ON) // middle
    {
      return;
    }
  }
}

// moves the cart to the left end of the track, causing leftwards motion until it is on the left side
void MaestroController::moveCartLeft(void)
{
  short cartPositions[1];
  short serials[] = {PRIMARY_SERIAL};
  short channels[] = {CART_END_0};
  getChannelValues(serials, channels, cartPositions, 1);
  ros::Rate delay(60);
  if (*cartPositions == ON_INP) // left end
  {
    return;
  }
  else
  {
    setTarget(PRIMARY_SERIAL, CART_LEFT, CART_0_MOTOR);
    while (*cartPositions != ON_INP)
    {
      getChannelValues(serials, channels, cartPositions, 1);
      delay.sleep();
    }
    setTarget(PRIMARY_SERIAL, STOP, CART_0_MOTOR);
    disableServo(PRIMARY_SERIAL, CART_0_MOTOR);
  }
}
// moves the cart to the right end of the track, causing rightwards motion until it is on the right side
void MaestroController::moveCartRight(void)
{
  /*short cartPositions[1];
   short serials[] = {PRIMARY_SERIAL};
   short channels[] = {CART_END_1};
   getChannelValues(serials, channels, cartPositions, 1);
   if (*cartPositions == ON) // right end
   {
   return;
   }
   else
   {
   setTarget(PRIMARY_SERIAL, CART_RIGHT, CART_0_MOTOR);
   while (*cartPositions == OFF)
   {
   getChannelValues(serials, channels, cartPositions, 1);
   }
   setTarget(PRIMARY_SERIAL, STOP, CART_0_MOTOR);
   }*/
}

// commands the scissor lift servo to move to the specified height
int MaestroController::moveScissorLift(short height, bool up)
{
  short serial[] = {PRIMARY_SERIAL};
  short channel[] = {CART_0_SERVO};
  short value[1];
  ros::Rate timer(30);
  if (up)
    value[0] = SCISSOR_MAX_UP;
  else
    value[0] = SCISSOR_MAX_DOWN;
  ROS_ERROR("Move scissor");
  ROS_ERROR("Sending value %d", value[0]);
  int ret = moveServos(serial, channel, value, 1);
  int count = 0;
  if (height == SCISSOR_DOWN)
  {
    channel[0] = 23;
    while (value[0] != ON_INP)
    {
      getChannelValues(serial, channel, value, 1);
      if (count == 3)
      {
        wiggle(timer);
        count = 0;
      }
      else
      {
        count++;
        timer.sleep();
        timer.sleep();
        timer.sleep();
        timer.sleep();
        timer.sleep();
        timer.sleep();
        timer.sleep();
        timer.sleep();
      }
    }
    disableServo(PRIMARY_SERIAL, CART_0_SERVO);
    return ret;
  }

  if (height == SCISSOR_UAV)
  {
    channel[0] = 22;
    while (value[0] != ON_INP)
    {
      getChannelValues(serial, channel, value, 1);
      if (count == 3)
      {
        wiggle(timer);
        count = 0;
      }
      else
      {
        count++;
        timer.sleep();
        timer.sleep();
        timer.sleep();
        timer.sleep();
        timer.sleep();
        timer.sleep();
        timer.sleep();
        timer.sleep();
      }
    }
    ros::Rate t_wait(10);
    t_wait.reset();
    t_wait.sleep();
    disableServo(PRIMARY_SERIAL, CART_0_SERVO);
    return ret;
  }

  channel[0] = SCISSOR_POT;
  long sum = 0;
  count = 1;
  bool in_pos = false;
  while (1)
  {
    getChannelValues(serial, channel, value, 1);
    if ((count % 10) == 0 && count != 0)
    {
      sum /= 10;
      int tar_pot = (int)((((float)height) * 0.0571));
      ROS_ERROR("target is %d, current avg is %ld", tar_pot, sum);
      if (sum >= tar_pot - 5 && sum <= tar_pot + 5)
      {
        ROS_ERROR("In range.");
        value[0] = height;
        channel[0] = CART_0_SERVO;
        ROS_ERROR("Slowing down scissor");
        ROS_ERROR("Sending value %d", value[0]);
        ret = moveServos(serial, channel, value, 1);

        ros::Rate delay(3);
        //tar_pot += 20;
        while (!in_pos)
        {
          disableServo(PRIMARY_SERIAL, CART_0_SERVO);
          delay.sleep();
          channel[0] = SCISSOR_POT;
          getChannelValues(serial, channel, value, 1);
          if (value[0] >= tar_pot - 5 && value[0] <= tar_pot + 5)
          {
            in_pos = true;
          }
          else
          {
            ROS_ERROR("Error is %d, value is %d, target is %d, outside dead band", value[0] - tar_pot, value[0],
                      tar_pot);
            int diff = tar_pot - value[0];
            ROS_ERROR("Difference is %d, height is %d, continuing to move", diff, height);
            value[0] = height + (diff * 50);
            channel[0] = CART_0_SERVO;
            ROS_ERROR("Sending value %d", value[0]);
            ret = moveServos(serial, channel, value, 1);
            wiggle(timer);
            delay.sleep();
            delay.sleep();
          }
        }
        ROS_ERROR("Disabling scissor");
        disableServo(PRIMARY_SERIAL, CART_0_SERVO);
        channel[0] = SCISSOR_POT;
        break;
      }
      sum = 0;
    }
    else
    {
      sum += value[0];
    }

    if (count == 20)
    {
      wiggle(timer);
      count = 1;
    }
    else
    {
      count++;
    }
  }
  return ret;
}
