/*
 * MaestroController_inputs.cpp
 *
 *  Created on: Sep 16, 2011
 *      Author: ra23501
 */

#include "MaestroController.hpp"

// obtains all the information for the channels on the specified Maestro controller(s), storing in the passed variables
// pre conditions: USB has been initialized and at least one Maestro device has been identified
void MaestroController::getChannelValues(short* serial, short* channel, short* positions, short numServos)
{
  MaestroVariables variables;
  variables.callStackPointer = 0;
  variables.errors = 0;
  variables.performanceFlags = 0;
  variables.programCounter = 0;
  variables.scriptDone = 0;
  variables.stackPointer = 0;
  ServoStatus* servos;
  if (serial[0] == MICRO_SERIAL)
  {
    servos = new ServoStatus[6];
  }
  else
  {
    servos = (ServoStatus*)calloc(24, sizeof(ServoStatus));
  }
  short stack[variables.stackPointer];
  short call_stack[variables.callStackPointer];
  bool serialSame = true;
  for (int i = 1; i < numServos; i++)
  {
    serialSame = (serial[i] == serial[i - 1]);
  }
  if (serialSame) // if the controllers specified are all the same, only one call must be made
  {
    getVariables(serial[0], variables, stack, call_stack, servos);
    for (int i = 0; i < numServos; i++)
    {
      positions[i] = servos[channel[i]].position;
    }
  }
  else // otherwise, make a call for each entry
  {
    for (int i = 0; i < numServos; i++)
    {
      getVariables(serial[i], variables, stack, call_stack, servos);
      positions[i] = servos[channel[i]].position;
    }
  }
}

// print out the current values for all the controller's variables to the screen
// pre conditions: USB has been initialized and at least one Maestro device has been identified
void MaestroController::displayStatus(short serial)
{
  MaestroVariables variables;
  variables.callStackPointer = 0;
  variables.errors = 0;
  variables.performanceFlags = 0;
  variables.programCounter = 0;
  variables.scriptDone = 0;
  variables.stackPointer = 0;
  ServoStatus* servos;
  if (serial == MICRO_SERIAL)
  {
    servos = new ServoStatus[6];
  }
  else
  {
    servos = (ServoStatus*)calloc(24, sizeof(ServoStatus));
  }
  short stack[variables.stackPointer];
  short call_stack[variables.callStackPointer];
  getVariables(serial, variables, stack, call_stack, servos);

  ROS_ERROR("#\ttarget\tspeed\taccel\tpos");
  if (serial == MICRO_SERIAL)
  {
    for (int i = 0; i < 6; i++)
    {
      ROS_ERROR("%d\t%hu\t%hu\t%hhu\t%hu", i, servos[i].target, servos[i].speed, servos[i].acceleration,
                servos[i].position);
    }
  }
  else
  {
    for (int i = 0; i < 24; i++)
    {
      ROS_ERROR("%d\t%hu\t%hu\t%hhu\t%hu", i, servos[i].target, servos[i].speed, servos[i].acceleration,
                servos[i].position);
    }
  }

  ROS_ERROR("errors: %d", variables.errors);

  ROS_ERROR("program counter: %d", variables.programCounter);
}

// function to get all variables from the controller
// individual calls differ based on the controller, which this handles by checking Mini/Micro
// pre conditions: USB has been initialized and at least one Maestro device has been identified
void MaestroController::getVariables(short serial, MaestroVariables variables, short* stack, short* callStack,
                                     ServoStatus* servos)
{
  if (serial == MICRO_SERIAL)
  {
    getVariablesMicroMaestro(serial, variables, stack, callStack, servos);
  }
  else
  {
    getVariablesMiniMaestro(serial, variables);
    getVariablesMiniMaestro(serial, servos);
    getVariablesMiniMaestro(serial, stack, true);
    getVariablesMiniMaestro(serial, callStack, false);
  }
}

// gets only the standard variables from the controller
// different calls for Mini/Micro
// pre conditions: USB has been initialized and at least one Maestro device has been identified
void MaestroController::getVariables(short serial, MaestroVariables variables)
{
  if (serial == MICRO_SERIAL)
  {
    ServoStatus servos[6];
    short stack[variables.stackPointer];
    short callStack[variables.callStackPointer];
    getVariablesMicroMaestro(serial, variables, stack, callStack, servos);
  }
  else
  {
    getVariablesMiniMaestro(serial, variables);
  }
}

// gets only the channel information from the controller
// different calls for Mini/Micro
// pre conditions: USB has been initialized and at least one Maestro device has been identified
void MaestroController::getVariables(short serial, ServoStatus* servos)
{
  if (serial == MICRO_SERIAL)
  {
    MaestroVariables variables;
    variables.callStackPointer = 0;
    variables.errors = 0;
    variables.performanceFlags = 0;
    variables.programCounter = 0;
    variables.scriptDone = 0;
    variables.stackPointer = 0;
    short stack[variables.stackPointer];
    short callStack[variables.callStackPointer];
    getVariablesMicroMaestro(serial, variables, stack, callStack, servos);
  }
  else
  {
    getVariablesMiniMaestro(serial, servos);
  }
}

// gets only the stack or callstack from the controller
// different calls for Mini/Micro
// boolean determines whether to get stack (true) or callstack (false)
// pre conditions: USB has been initialized and at least one Maestro device has been identified
void MaestroController::getVariables(short serial, short* stack, bool isStack)
{
  if (isStack) // stack case
  {
    if (serial == MICRO_SERIAL)
    {
      MaestroVariables variables;
      variables.callStackPointer = 0;
      variables.errors = 0;
      variables.performanceFlags = 0;
      variables.programCounter = 0;
      variables.scriptDone = 0;
      variables.stackPointer = 0;
      ServoStatus servos[6];
      short callStack[variables.callStackPointer];
      getVariablesMicroMaestro(serial, variables, stack, callStack, servos);
    }
    else
    {
      getVariablesMiniMaestro(serial, stack, true);
    }
  }
  else // callstack case
  {
    if (serial == MICRO_SERIAL)
    {
      MaestroVariables variables;
      variables.callStackPointer = 0;
      variables.errors = 0;
      variables.performanceFlags = 0;
      variables.programCounter = 0;
      variables.scriptDone = 0;
      variables.stackPointer = 0;
      ServoStatus servos[6];
      short callStack[variables.stackPointer];
      getVariablesMicroMaestro(serial, variables, callStack, stack, servos);
    }
    else
    {
      getVariablesMiniMaestro(serial, stack, false);
    }
  }
}

// queries the Micro controller for all variable information
// can only be used on the Micro, as the structures are a certain size
// pre conditions: USB has been initialized and at least one Maestro device has been identified
void MaestroController::getVariablesMicroMaestro(short serial, MaestroVariables variables, short* stack,
                                                 short* callStack, ServoStatus* servos)
{
  char* array = (char*)calloc(sizeof(MicroMaestroVariables), sizeof(char));

  controlTransfer(serial, 0xC0, REQUEST_GET_VARIABLES, 0, 0, array, sizeof(MicroMaestroVariables));

  char* pointer = array;
  MicroMaestroVariables tmp = *(MicroMaestroVariables*)pointer;
  variables.stackPointer = tmp.stackPointer;
  variables.callStackPointer = tmp.callStackPointer;
  variables.errors = tmp.errors;
  variables.programCounter = tmp.programCounter;
  variables.scriptDone = tmp.scriptDone;
  variables.performanceFlags = 0;

  for (int i = 0; i < 6; i++)
  {
    // the ServoStatus struct is rounded to 8 bytes by sizeof() instead of the 7 it takes up, must account for the error
    servos[i] = *((ServoStatus*)(pointer + sizeof(MicroMaestroVariables) - (sizeof(ServoStatus) * (6 - i)) - i));
  }

  stack = new short[variables.stackPointer];
  for (int i = 0; i < variables.stackPointer; i++)
  {
    stack[i] = *(tmp.stack + i);
  }

  callStack = new short[variables.callStackPointer];
  for (int i = 0; i < variables.callStackPointer; i++)
  {
    callStack[i] = *(tmp.callStack + i);
  }
}

// obtain just the variable information from the Mini controller
// pre conditions: USB has been initialized and at least one Maestro device has been identified
void MaestroController::getVariablesMiniMaestro(short serial, MaestroVariables variables)
{
  try
  {
    MiniMaestroVariables tmp;
    controlTransfer(serial, 0xC0, REQUEST_GET_VARIABLES, 0, 0, &tmp, (short)sizeof(MiniMaestroVariables));

    variables.stackPointer = tmp.stackPointer;
    variables.callStackPointer = tmp.callStackPointer;
    variables.errors = tmp.errors;
    variables.programCounter = tmp.programCounter;
    variables.scriptDone = tmp.scriptDone;
    variables.performanceFlags = tmp.performanceFlags;
  }
  catch (int e)
  {
    ROS_ERROR("Error getting variables from device.");
  }
}

// obtain just the channel information from the Mini controller
// pre conditions: USB has been initialized and at least one Maestro device has been identified
void MaestroController::getVariablesMiniMaestro(short serial, ServoStatus* servos)
{
  try
  {
    char* servoSettingsArray = (char*)calloc(SERVO_COUNT * sizeof(ServoStatus), sizeof(char));
    controlTransfer(serial, 0xC0, REQUEST_GET_SERVO_SETTINGS, 0, 0, servoSettingsArray, SERVO_COUNT
        * sizeof(ServoStatus));
    char* pointer = servoSettingsArray;
    for (int i = 0; i < SERVO_COUNT; i++)
    {
      // must account for sizeof() returning a rounded value (8 bytes) for ServoStatus (7 bytes)
      servos[i] = *((ServoStatus*)(pointer + ((sizeof(ServoStatus) - 1) * i)));
    }
  }
  catch (int e)
  {
    ROS_ERROR("Error getting channel settings from device.");
  }
}

// obtain just the stack or callstack information from the Mini controller
// stack (true) or callstack (false) is determined by the boolean isStack
// pre conditions: USB has been initialized and at least one Maestro device has been identified
void MaestroController::getVariablesMiniMaestro(short serial, short* stack, bool isStack)
{
  if (isStack) // stack case
  {
    try
    {
      stack = new short[MINI_MAESTRO_STACK_SIZE];
      short* pointer = stack;
      controlTransfer(serial, 0xC0, REQUEST_GET_STACK, 0, 0, pointer, (short)(sizeof(short) * MINI_MAESTRO_STACK_SIZE));
    }
    catch (int e)
    {
      ROS_ERROR("Error getting stack from device.");
    }
  }
  else // callstack case
  {
    try
    {
      stack = new short[MINI_MAESTRO_CALL_STACK_SIZE];
      short* pointer = stack;
      controlTransfer(serial, 0xC0, REQUEST_GET_CALL_STACK, 0, 0, pointer, (short)(sizeof(short)
          * MINI_MAESTRO_CALL_STACK_SIZE));
    }
    catch (int e)
    {
      ROS_ERROR("Error getting call stack from device.");
    }
  }
}
