/*
 * MaestroController_stepper.cpp
 *
 *  Created on: Sep 20, 2011
 *      Author: ra23501
 */

#include "MaestroController.hpp"

// move the stepper to the home position
void MaestroController::homeStepper()
{
  stepperAt = getStepperLocation();
  if (stepperAt == STEPPER_HOME - 11)
  {
    ROS_ERROR("At home");
    return;
  }
  else if (stepperAt == NUM_BATTERIES)
  {
    ROS_ERROR("Not home, moving CCW 1 to find heading");
    while (stepperAt == NUM_BATTERIES)
    {
      shiftCCW();
      stepperAt = getStepperLocation();
      if (stepperAt == 0)
      {
        shiftCW();
        shiftCW();
        shiftCW();
        while ((stepperAt = getStepperLocation()) == NUM_BATTERIES)
        {
          shiftCW();
        }
      }
    }
    ROS_ERROR("Currently at %d, moving to home", stepperAt);
    moveToStation(STEPPER_HOME - 11);
  }
  else
  {
    ROS_ERROR("Currently at %d, moving to home", stepperAt);
    moveToStation(STEPPER_HOME - 11);
  }
  stepperAt = STEPPER_HOME - 11;
}

// move the stepper to the desired station, given current and target
// optimizes movement for limits of CCW motion on station 0 and CW motion on station 7
// assuming cart can move to either end of the track as well
void MaestroController::moveToStation(char target)
{
  if (stepperAt == target)
    return;

  int change = target - stepperAt;
  ROS_ERROR("Need to move %d stations", change);

  if (change < 0)
  {
    moveCCW(target);
  }
  else
  {
    moveCW(target);
  }

  stepperAt = target;
}

// obtain the current location of the stepper motor based on photo interrupters
// returns the current location or an out of bounds value if not at a single location
char MaestroController::getStepperLocation()
{
  short serials[] = {PRIMARY_SERIAL, PRIMARY_SERIAL, PRIMARY_SERIAL, PRIMARY_SERIAL, PRIMARY_SERIAL, PRIMARY_SERIAL,
                     PRIMARY_SERIAL, PRIMARY_SERIAL};
  short channels[] = {BATTERY_STATION_0, BATTERY_STATION_1, BATTERY_STATION_2, BATTERY_STATION_3, BATTERY_STATION_4,
                      BATTERY_STATION_5, BATTERY_STATION_6, BATTERY_STATION_7};

  short positions[NUM_BATTERIES];
  getChannelValues(serials, channels, positions, NUM_BATTERIES);
  for (int i = 0; i < NUM_BATTERIES; i++)
  {
    if (positions[i] == ON_INP)
    {
      return i;
    }
  }

  return NUM_BATTERIES;
}

// move the stepper motor counter clockwise to a certain station
void MaestroController::moveCCW(char dest)
{
  // set direction to CCW
  ros::Rate timer(60);
  int count = 0;
  ROS_ERROR("Moving CCW");
  controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, STEPPER_CCW, STEPPER_DIRECTION);
  while (getStepperLocation() != dest)
  {
    if (count == 2)
    {
      controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, OFF, STEPPER_STEPS);
      timer.sleep();
      controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, ON, STEPPER_STEPS);
      timer.sleep();
      count = 0;
    }
    else
    {
      timer.sleep();
      timer.sleep();
      count++;
    }
  }

  ros::Duration(0.3).sleep();
  controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, STEPPER_CW, STEPPER_DIRECTION);
  while (getStepperLocation() != dest)
  {
    controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, OFF, STEPPER_STEPS);
    timer.sleep();
    controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, ON, STEPPER_STEPS);
    timer.sleep();
    timer.sleep();
    timer.sleep();
    timer.sleep();
    timer.sleep();
    timer.sleep();
    timer.sleep();
    timer.sleep();
  }
}

// move the stepper motor counter clockwise to a certain station
void MaestroController::moveCW(char dest)
{
  ros::Rate timer(60);
  int count = 0;
  ROS_ERROR("Moving CW");
  controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, STEPPER_CW, STEPPER_DIRECTION);
  while (getStepperLocation() != dest)
  {
    if (count == 2)
    {
      controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, OFF, STEPPER_STEPS);
      timer.sleep();
      controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, ON, STEPPER_STEPS);
      timer.sleep();
      count = 0;
    }
    else
    {
      timer.sleep();
      timer.sleep();
      count++;
    }
  }

  ros::Duration(0.3).sleep();
  controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, STEPPER_CCW, STEPPER_DIRECTION);
  while (getStepperLocation() != dest)
  {
    controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, OFF, STEPPER_STEPS);
    timer.sleep();
    controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, ON, STEPPER_STEPS);
    timer.sleep();
    timer.sleep();
    timer.sleep();
    timer.sleep();
    timer.sleep();
    timer.sleep();
    timer.sleep();
    timer.sleep();
  }
}

void MaestroController::wiggle(ros::Rate timer)
{
  /*controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, STEPPER_CW, STEPPER_DIRECTION);
   controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, OFF, STEPPER_STEPS);
   timer.sleep();
   controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, ON, STEPPER_STEPS);
   timer.sleep();
   controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, OFF, STEPPER_STEPS);
   timer.sleep();
   controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, ON, STEPPER_STEPS);
   timer.sleep();
   controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, OFF, STEPPER_STEPS);
   timer.sleep();
   controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, ON, STEPPER_STEPS);
   timer.sleep();
   moveCCW(stepperAt);
   controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, STEPPER_CCW, STEPPER_DIRECTION);
   controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, OFF, STEPPER_STEPS);
   timer.sleep();
   controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, ON, STEPPER_STEPS);
   timer.sleep();
   controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, OFF, STEPPER_STEPS);
   timer.sleep();
   controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, ON, STEPPER_STEPS);
   timer.sleep();
   controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, OFF, STEPPER_STEPS);
   timer.sleep();
   controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, ON, STEPPER_STEPS);
   timer.sleep();
   moveCW(stepperAt);*/
  controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, STEPPER_CW, STEPPER_DIRECTION);
  controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, OFF, STEPPER_STEPS);
  timer.sleep();
  controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, ON, STEPPER_STEPS);
  timer.sleep();
  controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, OFF, STEPPER_STEPS);
  timer.sleep();
  controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, ON, STEPPER_STEPS);
  timer.sleep();
  controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, STEPPER_CCW, STEPPER_DIRECTION);
  controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, OFF, STEPPER_STEPS);
  timer.sleep();
  controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, ON, STEPPER_STEPS);
  timer.sleep();
  controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, OFF, STEPPER_STEPS);
  timer.sleep();
  controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, ON, STEPPER_STEPS);
  timer.sleep();
  controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, OFF, STEPPER_STEPS);
  timer.sleep();
  controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, ON, STEPPER_STEPS);
  timer.sleep();
  controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, OFF, STEPPER_STEPS);
  timer.sleep();
  controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, ON, STEPPER_STEPS);
  timer.sleep();
  controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, STEPPER_CW, STEPPER_DIRECTION);
  controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, OFF, STEPPER_STEPS);
  timer.sleep();
  controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, ON, STEPPER_STEPS);
  timer.sleep();
  controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, OFF, STEPPER_STEPS);
  timer.sleep();
  controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, ON, STEPPER_STEPS);
  timer.sleep();
}

void MaestroController::shiftCW(void)
{
  ros::Rate timer(30);
  controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, STEPPER_CW, STEPPER_DIRECTION);
  controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, OFF, STEPPER_STEPS);
  timer.sleep();
  controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, ON, STEPPER_STEPS);
  timer.sleep();
}

void MaestroController::shiftCCW(void)
{
  ros::Rate timer(30);
  controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, STEPPER_CCW, STEPPER_DIRECTION);
  controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, OFF, STEPPER_STEPS);
  timer.sleep();
  controlTransfer(PRIMARY_SERIAL, LIBUSB_REQUEST_TYPE_VENDOR, REQUEST_SET_TARGET, ON, STEPPER_STEPS);
  timer.sleep();
}
