/**
 * \file pelican_simulator.cpp
 *
 * \author Brian J. Julian
 *
 * \version 0.1
 *
 * \date 30 Aug 2011
 *
 */


// includes
#include <pelican_simulator.h>


// defines
#define ROS_PKG "pelican_simulator"
#define ROS_NODE "pelican_simulator"


// class constructor
PelicanSimulator::PelicanSimulator(void)
  : nh_(),
    private_nh_("~")
{
  // init params
  initParams();

  // init publishers
  initPublishers();

  // init subscribers
  initSubscribers();

  // start pelican thread
  startPelicanThread();

  // start publishing robot state
  startRobotStatePubThread();
}


// class destructor
PelicanSimulator::~PelicanSimulator(void)
{
  // stop all threads
  stopPelicanThread();
  stopRobotStatePubThread();
}


// main function
int main(int argc, char **argv)
{
  // initialize ros node handle
  ros::init(argc, argv, ROS_NODE);

  // print start to log file
  ROS_INFO("Started node %s", ROS_NODE);

  // create class instance
  PelicanSimulator pelican_simulator;

  // ros spin
  ros::spin();

  // print termination to log file
  ROS_INFO("Stopped node %s", ROS_NODE);

  // return success
  return(0);           
}
