/**
 * \file pelican_simulator_init.cpp
 *
 * \author Brian J. Julian
 *
 * \version 0.1
 *
 * \date 30 Aug 2011
 *
 */


// includes
#include <pelican_simulator.h>


// init params
void PelicanSimulator::initParams(void)
{
  // ros params
  private_nh_.param("uid", uid_, string("robot_name"));
  private_nh_.param("robot_state_pub_rate", robot_state_pub_rate_, 1.0);
  private_nh_.param("pelican_thread_rate", pelican_thread_rate_, 20.0);
  private_nh_.param("flight_height", flight_height_, 10.0);
  private_nh_.param("flight_speed", flight_speed_, 1.0);
  private_nh_.param("max_battery_voltage", max_battery_voltage_, 12.6);
  private_nh_.param("min_battery_voltage", min_battery_voltage_, 9.5);
  private_nh_.param("return_battery_voltage", return_battery_voltage_, 9.9);
  private_nh_.param("battery_charge_rate", battery_charge_rate_, 0.5);
  private_nh_.param("battery_discharge_rate", battery_discharge_rate_, 0.1);
  
  // init params
  robot_state_.uid = uid_;
  robot_state_.translation = geometry_msgs::Vector3();
  robot_state_.rotation = geometry_msgs::Quaternion();
  robot_state_.rotation.w = 1.0;
  robot_state_.battery_voltage = return_battery_voltage_;
  pelican_state_ = CHARGING;

  // init volatile bools
  received_land_request_ = false;
  received_positive_land_confirm_ = false;
  received_negative_land_confirm_ = false;
}


// init publishers
void PelicanSimulator::initPublishers(void)
{
  robot_state_pub_ = nh_.advertise<wpi_msgs::RobotState>("robot_state", 10, true);
  land_request_pub_ = nh_.advertise<wpi_msgs::LandRequest>("land_request", 10, true);
  land_confirm_pub_ = nh_.advertise<wpi_msgs::LandConfirm>("land_confirm", 10, true);
}


// init subscribers
void PelicanSimulator::initSubscribers(void)
{
  land_request_sub_ = nh_.subscribe(uid_ + string("/land_request"), 10, &PelicanSimulator::callbackLandRequestMsg, this);
  land_confirm_sub_ = nh_.subscribe(uid_ + string("/land_confirm"), 10, &PelicanSimulator::callbackLandConfirmMsg, this);
}
