/**
 * \file pelican_simulator_thread.cpp
 *
 * \author Brian J. Julian
 *
 * \version 0.1
 *
 * \date 30 Aug 2011
 *
 */


// includes
#include <pelican_simulator.h>


// start pelican thread
void PelicanSimulator::startPelicanThread(void)
{
  pelican_thread_ = boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&PelicanSimulator::pelicanThread, this)));
}


// pelican thread
void PelicanSimulator::pelicanThread(void)
{
  // set thread rate
  ros::Rate thread_rate(pelican_thread_rate_);

  // while node active
  while(ros::ok())
    {
      // pelican state machine
      mutex_.lock();
      switch(pelican_state_)
	{
	case(CHARGING):
	  chargingState();
	  break;
	case(LAUNCHING):
	  launchingState();
	  break;
	case(FLYING):
	  flyingState();
	  break;
	case(FLYING_WITH_LOW_BATTERY):
	  flyingWithLowBatteryState();
	  break;
	case(LANDING):
	  landingState();
	  break;
	default:
	  break;
	}
      mutex_.unlock();

      // regulate thread rate
      thread_rate.sleep();
    }
}


// stop pelican thread
void PelicanSimulator::stopPelicanThread(void)
{
  pelican_thread_->join();
}


// start robot state pub thread
void PelicanSimulator::startRobotStatePubThread(void)
{
  robot_state_pub_thread_ = boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&PelicanSimulator::robotStatePubThread, this)));
}


// robot state pub thread
void PelicanSimulator::robotStatePubThread(void)
{
  // set thread rate
  ros::Rate thread_rate(robot_state_pub_rate_);

  // while node active
  while(ros::ok())
    {
      // publish robot state
      mutex_.lock();
      robot_state_.header.stamp = ros::Time::now();
      robot_state_pub_.publish(robot_state_);
      mutex_.unlock();

      // regulate thread rate
      thread_rate.sleep();
    }
}

// stop robot state pub thread
void PelicanSimulator::stopRobotStatePubThread(void)
{
  robot_state_pub_thread_->join();
}
